@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2018 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@
 #include "nx_session_manager.h"
 
 #define REMMINA_PLUGIN_NX_FEATURE_TOOL_SENDCTRLALTDEL 1
+#define REMMINA_PLUGIN_NX_FEATURE_GTKSOCKET 1
 
 /* Forward declaration */
 static RemminaProtocolPlugin remmina_plugin_nx;
@@ -581,7 +582,7 @@ static gboolean remmina_plugin_nx_open_connection(RemminaProtocolWidget *gp)
 
 	if (!remmina_plugin_nx_service->gtksocket_available()) {
 		remmina_plugin_nx_service->protocol_plugin_set_error(gp,
-			_("Protocol %s is unavailable because GtkSocket only works under Xorg"),
+			_("Protocol %s is unavailable because GtkSocket only works under X.Org"),
 			remmina_plugin_nx.name);
 		return FALSE;
 	}
@@ -597,7 +598,7 @@ static gboolean remmina_plugin_nx_open_connection(RemminaProtocolWidget *gp)
 
 	if (pthread_create(&gpdata->thread, NULL, remmina_plugin_nx_main_thread, gp)) {
 		remmina_plugin_nx_service->protocol_plugin_set_error(gp,
-			"Failed to initialize pthread. Falling back to non-thread mode...");
+			"Failed to initialize pthread. Falling back to non-thread mode…");
 		gpdata->thread = 0;
 		return FALSE;
 	}else  {
@@ -725,6 +726,7 @@ static const RemminaProtocolSetting remmina_plugin_nx_advanced_settings[] =
 static const RemminaProtocolFeature remmina_plugin_nx_features[] =
 {
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_NX_FEATURE_TOOL_SENDCTRLALTDEL, N_("Send Ctrl+Alt+Delete"), NULL, NULL },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_GTKSOCKET, REMMINA_PLUGIN_NX_FEATURE_GTKSOCKET,	    NULL,	   NULL,	NULL},
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_END,  0,					     NULL,			 NULL, NULL }
 };
 
@@ -736,8 +738,8 @@ static RemminaProtocolPlugin remmina_plugin_nx =
 	N_("NX - NX Technology"),                       // Description
 	GETTEXT_PACKAGE,                                // Translation domain
 	VERSION,                                        // Version number
-	"remmina-nx",                                   // Icon for normal connection
-	"remmina-nx",                                   // Icon for SSH connection
+	"remmina-nx-symbolic",                          // Icon for normal connection
+	"remmina-nx-symbolic",                          // Icon for SSH connection
 	remmina_plugin_nx_basic_settings,               // Array for basic settings
 	remmina_plugin_nx_advanced_settings,            // Array for advanced settings
 	REMMINA_PROTOCOL_SSH_SETTING_TUNNEL,            // SSH settings type
