@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2018 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +58,12 @@
 
 #include <string.h>
 
+#ifdef GDK_WINDOWING_X11
+	#include <X11/Xlib.h>
+	#include <X11/XKBlib.h>
+	#include <gdk/gdkx.h>
+#endif
+
 #define REMMINA_RDP_FEATURE_TOOL_REFRESH         1
 #define REMMINA_RDP_FEATURE_SCALE                2
 #define REMMINA_RDP_FEATURE_UNFOCUS              3
@@ -80,6 +86,7 @@ static BOOL rf_process_event_queue(RemminaProtocolWidget* gp)
 	rfContext* rfi = GET_PLUGIN_DATA(gp);
 	RemminaPluginRdpEvent* event;
 	DISPLAY_CONTROL_MONITOR_LAYOUT* dcml;
+	CLIPRDR_FORMAT_DATA_RESPONSE response = { 0 };
 
 	if (rfi->event_queue == NULL)
 		return True;
@@ -117,10 +124,10 @@ static BOOL rf_process_event_queue(RemminaProtocolWidget* gp)
 			break;
 
 		case REMMINA_RDP_EVENT_TYPE_CLIPBOARD_SEND_CLIENT_FORMAT_DATA_RESPONSE:
-			rfi->clipboard.context->ClientFormatDataResponse(rfi->clipboard.context, event->clipboard_formatdataresponse.pFormatDataResponse);
-			if (event->clipboard_formatdataresponse.pFormatDataResponse->requestedFormatData)
-				free(event->clipboard_formatdataresponse.pFormatDataResponse->requestedFormatData);
-			free(event->clipboard_formatdataresponse.pFormatDataResponse);
+			response.msgFlags = (event->clipboard_formatdataresponse.data) ? CB_RESPONSE_OK : CB_RESPONSE_FAIL;
+			response.dataLen = event->clipboard_formatdataresponse.size;
+			response.requestedFormatData = event->clipboard_formatdataresponse.data;
+			rfi->clipboard.context->ClientFormatDataResponse(rfi->clipboard.context, &response);
 			break;
 
 		case REMMINA_RDP_EVENT_TYPE_CLIPBOARD_SEND_CLIENT_FORMAT_DATA_REQUEST:
@@ -247,7 +254,7 @@ BOOL rf_auto_reconnect(rfContext* rfi)
 	/* Sleep half a second to allow:
 	 *  - processing of the ui event we just pushed on the queue
 	 *  - better network conditions
-	 *  Remember: we hare on a thread, so the main gui won't lock */
+	 *  Remember: we hare on a thread, so the main gui won’t lock */
 
 	usleep(500000);
 
@@ -297,7 +304,6 @@ BOOL rf_begin_paint(rdpContext* context)
 {
 	TRACE_CALL(__func__);
 	rdpGdi* gdi;
-	HGDI_WND hwnd;
 
 	if (!context)
 		return FALSE;
@@ -306,44 +312,50 @@ BOOL rf_begin_paint(rdpContext* context)
 	if (!gdi || !gdi->primary || !gdi->primary->hdc || !gdi->primary->hdc->hwnd)
 		return FALSE;
 
-	hwnd = gdi->primary->hdc->hwnd;
-	if (!hwnd->ninvalid)
-		return FALSE;
-
-	hwnd->invalid->null = 1;
-	hwnd->ninvalid = 0;
 	return TRUE;
 }
 
 BOOL rf_end_paint(rdpContext* context)
 {
 	TRACE_CALL(__func__);
-	INT32 x, y;
-	UINT32 w, h;
 	rdpGdi* gdi;
 	rfContext* rfi;
 	RemminaPluginRdpUiObject* ui;
+	int i, ninvalid;
+	region *reg;
+	HGDI_RGN cinvalid;
 
 	gdi = context->gdi;
 	rfi = (rfContext*)context;
 
 	if (gdi->primary->hdc->hwnd->invalid->null)
-		return FALSE;
+		return TRUE;
 
-	x = gdi->primary->hdc->hwnd->invalid->x;
-	y = gdi->primary->hdc->hwnd->invalid->y;
-	w = gdi->primary->hdc->hwnd->invalid->w;
-	h = gdi->primary->hdc->hwnd->invalid->h;
+	if (gdi->primary->hdc->hwnd->ninvalid < 1)
+		return TRUE;
+
+	ninvalid = gdi->primary->hdc->hwnd->ninvalid;
+	cinvalid = gdi->primary->hdc->hwnd->cinvalid;
+	reg = (region *)g_malloc(sizeof(region) * ninvalid);
+	for(i = 0; i < ninvalid; i++) {
+		reg[i].x = cinvalid[i].x;
+		reg[i].y = cinvalid[i].y;
+		reg[i].w = cinvalid[i].w;
+		reg[i].h = cinvalid[i].h;
+	}
 
 	ui = g_new0(RemminaPluginRdpUiObject, 1);
-	ui->type = REMMINA_RDP_UI_UPDATE_REGION;
-	ui->region.x = x;
-	ui->region.y = y;
-	ui->region.width = w;
-	ui->region.height = h;
+	ui->type = REMMINA_RDP_UI_UPDATE_REGIONS;
+	ui->reg.ninvalid = ninvalid;
+	ui->reg.ureg = reg;
 
 	remmina_rdp_event_queue_ui_async(rfi->protocol_widget, ui);
 
+
+	gdi->primary->hdc->hwnd->invalid->null = TRUE;
+	gdi->primary->hdc->hwnd->ninvalid = 0;
+
+
 	return TRUE;
 }
 
@@ -360,7 +372,17 @@ static BOOL rf_desktop_resize(rdpContext* context)
 	remmina_plugin_service->protocol_plugin_set_width(gp, rfi->settings->DesktopWidth);
 	remmina_plugin_service->protocol_plugin_set_height(gp, rfi->settings->DesktopHeight);
 
-	/* Call to remmina_rdp_event_update_scale(gp) on the main UI thread */
+	ui = g_new0(RemminaPluginRdpUiObject, 1);
+	ui->type = REMMINA_RDP_UI_EVENT;
+	ui->event.type = REMMINA_RDP_UI_EVENT_DESTROY_CAIRO_SURFACE;
+	remmina_rdp_event_queue_ui_sync_retint(gp, ui);
+
+	/* Tell libfreerdp to change its internal GDI bitmap width and heigt,
+	 * this will also destroy gdi->primary_buffer, making our rfi->surface invalid */
+	gdi_resize(((rdpContext*)rfi)->gdi, rfi->settings->DesktopWidth, rfi->settings->DesktopHeight);
+
+	/* Call to remmina_rdp_event_update_scale(gp) on the main UI thread,
+	 * this will recreate rfi->surface from gdi->primary_buffer */
 
 	ui = g_new0(RemminaPluginRdpUiObject, 1);
 	ui->type = REMMINA_RDP_UI_EVENT;
@@ -372,6 +394,64 @@ static BOOL rf_desktop_resize(rdpContext* context)
 	return TRUE;
 }
 
+static BOOL rf_play_sound(rdpContext* context, const PLAY_SOUND_UPDATE* play_sound)
+{
+	TRACE_CALL(__func__);
+	rfContext* rfi;
+	RemminaProtocolWidget* gp;
+	GdkDisplay* disp;
+
+	rfi = (rfContext*)context;
+	gp = rfi->protocol_widget;
+
+	disp = gtk_widget_get_display(GTK_WIDGET(gp));
+	gdk_display_beep(disp);
+
+	return TRUE;
+}
+
+static BOOL rf_keyboard_set_indicators(rdpContext* context, UINT16 led_flags)
+{
+	TRACE_CALL(__func__);
+	rfContext* rfi;
+	RemminaProtocolWidget* gp;
+	GdkDisplay* disp;
+
+	rfi = (rfContext*)context;
+	gp = rfi->protocol_widget;
+	disp = gtk_widget_get_display(GTK_WIDGET(gp));
+
+#ifdef GDK_WINDOWING_X11
+	if (GDK_IS_X11_DISPLAY(disp)) {
+		/* ToDo: we are not on the main thread. Will X.Org complain ? */
+		Display* x11_display;
+		x11_display = gdk_x11_display_get_xdisplay(disp);
+		XkbLockModifiers(x11_display, XkbUseCoreKbd,
+			LockMask | Mod2Mask,
+			(led_flags & KBD_SYNC_CAPS_LOCK ? LockMask : 0) |
+			(led_flags & KBD_SYNC_NUM_LOCK ? Mod2Mask : 0)
+			);
+
+		/* ToDo: add support to KANA_LOCK and SCROLL_LOCK */
+	}
+#endif
+
+	return TRUE;
+}
+
+BOOL rf_keyboard_set_ime_status(rdpContext* context, UINT16 imeId, UINT32 imeState,
+                                UINT32 imeConvMode)
+{
+	TRACE_CALL(__func__);
+	if (!context)
+			return FALSE;
+
+	/* Unimplemented, we ignore it */
+
+	return TRUE;
+}
+
+
 static BOOL remmina_rdp_pre_connect(freerdp* instance)
 {
 	TRACE_CALL(__func__);
@@ -383,37 +463,10 @@ static BOOL remmina_rdp_pre_connect(freerdp* instance)
 
 	settings->OsMajorType = OSMAJORTYPE_UNIX;
 	settings->OsMinorType = OSMINORTYPE_UNSPECIFIED;
-	ZeroMemory(settings->OrderSupport, 32);
 
 	settings->BitmapCacheEnabled = True;
 	settings->OffscreenSupportLevel = True;
 
-
-	settings->OrderSupport[NEG_DSTBLT_INDEX] = True;
-	settings->OrderSupport[NEG_PATBLT_INDEX] = True;
-	settings->OrderSupport[NEG_SCRBLT_INDEX] = True;
-	settings->OrderSupport[NEG_OPAQUE_RECT_INDEX] = True;
-	settings->OrderSupport[NEG_DRAWNINEGRID_INDEX] = False;
-	settings->OrderSupport[NEG_MULTIDSTBLT_INDEX] = False;
-	settings->OrderSupport[NEG_MULTIPATBLT_INDEX] = False;
-	settings->OrderSupport[NEG_MULTISCRBLT_INDEX] = False;
-	settings->OrderSupport[NEG_MULTIOPAQUERECT_INDEX] = True;
-	settings->OrderSupport[NEG_MULTI_DRAWNINEGRID_INDEX] = False;
-	settings->OrderSupport[NEG_LINETO_INDEX] = True;
-	settings->OrderSupport[NEG_POLYLINE_INDEX] = True;
-	settings->OrderSupport[NEG_MEMBLT_INDEX] = settings->BitmapCacheEnabled;
-	settings->OrderSupport[NEG_MEM3BLT_INDEX] = settings->BitmapCacheEnabled;
-	settings->OrderSupport[NEG_MEMBLT_V2_INDEX] = settings->BitmapCacheEnabled;
-	settings->OrderSupport[NEG_MEM3BLT_V2_INDEX] = settings->BitmapCacheEnabled;
-	settings->OrderSupport[NEG_SAVEBITMAP_INDEX] = False;
-	settings->OrderSupport[NEG_GLYPH_INDEX_INDEX] = True;
-	settings->OrderSupport[NEG_FAST_INDEX_INDEX] = True;
-	settings->OrderSupport[NEG_FAST_GLYPH_INDEX] = True;
-	settings->OrderSupport[NEG_POLYGON_SC_INDEX] = False;
-	settings->OrderSupport[NEG_POLYGON_CB_INDEX] = False;
-	settings->OrderSupport[NEG_ELLIPSE_SC_INDEX] = False;
-	settings->OrderSupport[NEG_ELLIPSE_CB_INDEX] = False;
-
 	if (settings->RemoteFxCodec == True) {
 		settings->FrameAcknowledge = False;
 		settings->LargePointerFlag = True;
@@ -438,15 +491,12 @@ static BOOL remmina_rdp_post_connect(freerdp* instance)
 	rfContext* rfi;
 	RemminaProtocolWidget* gp;
 	RemminaPluginRdpUiObject* ui;
-	rdpGdi* gdi;
 	UINT32 freerdp_local_color_format;
 
 	rfi = (rfContext*)instance->context;
 	gp = rfi->protocol_widget;
 	rfi->postconnect_error = REMMINA_POSTCONNECT_ERROR_OK;
 
-	rfi->width = rfi->settings->DesktopWidth;
-	rfi->height = rfi->settings->DesktopHeight;
 	rfi->srcBpp = rfi->settings->ColorDepth;
 
 	if (rfi->settings->RemoteFxCodec == FALSE)
@@ -458,7 +508,7 @@ static BOOL remmina_rdp_post_connect(freerdp* instance)
 		freerdp_local_color_format = PIXEL_FORMAT_BGRA32;
 		rfi->cairo_format = CAIRO_FORMAT_ARGB32;
 	}else if (rfi->bpp == 24) {
-		/* CAIRO_FORMAT_RGB24 is 32bit aligned, so we map it to libfreerdp's PIXEL_FORMAT_BGRX32 */
+		/* CAIRO_FORMAT_RGB24 is 32bit aligned, so we map it to libfreerdp’s PIXEL_FORMAT_BGRX32 */
 		freerdp_local_color_format = PIXEL_FORMAT_BGRX32;
 		rfi->cairo_format = CAIRO_FORMAT_RGB24;
 	}else {
@@ -477,15 +527,16 @@ static BOOL remmina_rdp_post_connect(freerdp* instance)
 		return FALSE;
 	}
 
-	gdi = instance->context->gdi;
-	rfi->primary_buffer = gdi->primary_buffer;
-
 	pointer_cache_register_callbacks(instance->update);
 
 	instance->update->BeginPaint = rf_begin_paint;
 	instance->update->EndPaint = rf_end_paint;
 	instance->update->DesktopResize = rf_desktop_resize;
 
+	instance->update->PlaySound = rf_play_sound;
+	instance->update->SetKeyboardIndicators = rf_keyboard_set_indicators;
+	instance->update->SetKeyboardImeStatus = rf_keyboard_set_ime_status;
+
 	remmina_rdp_clipboard_init(rfi);
 	rfi->connected = True;
 
@@ -527,7 +578,7 @@ static BOOL remmina_rdp_authenticate(freerdp* instance, char** username, char**
 		if (save) {
 			// User has requested to save credentials. We put all the new cretentials
 			// into remminafile->settings. They will be saved later, on successful connection, by
-			// remmina_connection_window.c
+			// rcw.c
 
 			remmina_plugin_service->file_set_string( remminafile, "username", s_username );
 			remmina_plugin_service->file_set_string( remminafile, "password", s_password );
@@ -548,6 +599,62 @@ static BOOL remmina_rdp_authenticate(freerdp* instance, char** username, char**
 	return True;
 }
 
+static BOOL remmina_rdp_gw_authenticate(freerdp* instance, char** username, char** password, char** domain)
+{
+	TRACE_CALL(__func__);
+	gchar *s_username, *s_password, *s_domain;
+	gint ret;
+	rfContext* rfi;
+	RemminaProtocolWidget* gp;
+	gboolean save;
+	gboolean disablepasswordstoring;
+	RemminaFile* remminafile;
+
+	rfi = (rfContext*)instance->context;
+	gp = rfi->protocol_widget;
+	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
+
+	if (!remmina_plugin_service->file_get_string(remminafile, "gateway_server"))
+		return False;
+	disablepasswordstoring = remmina_plugin_service->file_get_int(remminafile, "disablepasswordstoring", FALSE);
+	ret = remmina_plugin_service->protocol_plugin_init_authuserpwd(gp, TRUE, !disablepasswordstoring);
+
+	if (ret == GTK_RESPONSE_OK) {
+		s_username = remmina_plugin_service->protocol_plugin_init_get_username(gp);
+		if (s_username) rfi->settings->GatewayUsername  = strdup(s_username);
+
+		s_password = remmina_plugin_service->protocol_plugin_init_get_password(gp);
+		if (s_password) rfi->settings->GatewayPassword = strdup(s_password);
+
+		s_domain = remmina_plugin_service->protocol_plugin_init_get_domain(gp);
+		if (s_domain) rfi->settings->GatewayDomain = strdup(s_domain);
+
+		save = remmina_plugin_service->protocol_plugin_init_get_savepassword(gp);
+		if (save) {
+			// User has requested to save credentials. We put all the new cretentials
+			// into remminafile->settings. They will be saved later, on successful connection, by
+			// rcw.c
+
+			remmina_plugin_service->file_set_string( remminafile, "gateway_username", s_username );
+			remmina_plugin_service->file_set_string( remminafile, "gateway_password", s_password );
+			remmina_plugin_service->file_set_string( remminafile, "gateway_domain", s_domain );
+
+		}
+
+		if ( s_username ) g_free( s_username );
+		if ( s_password ) g_free( s_password );
+		if ( s_domain ) g_free( s_domain );
+
+		return True;
+	}else {
+		rfi->user_cancelled = TRUE;
+		return False;
+	}
+
+	return True;
+}
+
+
 static DWORD remmina_rdp_verify_certificate(freerdp* instance, const char *common_name, const char* subject,
 					    const char* issuer, const char* fingerprint, BOOL host_mismatch)
 {
@@ -628,7 +735,8 @@ static void remmina_rdp_main_loop(RemminaProtocolWidget* gp)
 				remmina_plugin_service->protocol_plugin_set_error(gp, NULL);
 				continue;
 			}
-			fprintf(stderr, "Failed to check FreeRDP event handles\n");
+			if (freerdp_get_last_error(rfi->instance->context) == FREERDP_ERROR_SUCCESS)
+				fprintf(stderr, "Failed to check FreeRDP file descriptor\n");
 			break;
 		}
 	}
@@ -676,18 +784,27 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 	const gchar* cs;
 	RemminaFile* remminafile;
 	rfContext* rfi = GET_PLUGIN_DATA(gp);
+	rdpChannels* channels;
 	gchar *gateway_host;
 	gint gateway_port;
 	gint desktopOrientation, desktopScaleFactor, deviceScaleFactor;
-	gint dynresw, dynresh;
+
+	channels = rfi->instance->context->channels;
 
 	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
 
+#if defined(PROXY_TYPE_IGNORE)
+	if ( !remmina_plugin_service->file_get_int(remminafile, "enableproxy", FALSE) ? TRUE : FALSE ) {
+		remmina_plugin_service->log_print("[RDP] Ignoring proxy environment variables\n");
+		rfi->settings->ProxyType = PROXY_TYPE_IGNORE;
+	}
+#endif
+
 	if (!remmina_rdp_tunnel_init(gp))
 		return FALSE;
 
 	rfi->settings->AutoReconnectionEnabled = ( remmina_plugin_service->file_get_int(remminafile, "disableautoreconnect", FALSE) ? FALSE : TRUE );
-	/* Disable RDP auto reconnection when ssh tunnel is enabled */
+	/* Disable RDP auto reconnection when SSH tunnel is enabled */
 	if (remmina_plugin_service->file_get_int(remminafile, "ssh_enabled", FALSE)) {
 		rfi->settings->AutoReconnectionEnabled = FALSE;
 	}
@@ -696,7 +813,7 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 
 	rfi->settings->SoftwareGdi = TRUE;
 
-	/* Avoid using H264 modes if they are not available on libfreerdp */
+	/* Avoid using H.264 modes if they are not available on libfreerdp */
 	if (!gfx_h264_available && (rfi->settings->ColorDepth == 65 || rfi->settings->ColorDepth == 66))
 		rfi->settings->ColorDepth = 64;	// Fallback to GFX RFX
 
@@ -727,13 +844,22 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 
 	rfi->settings->DesktopWidth = remmina_plugin_service->get_profile_remote_width(gp);
 	rfi->settings->DesktopHeight = remmina_plugin_service->get_profile_remote_height(gp);
-	dynresw = remmina_plugin_service->file_get_int(remminafile, "dynamic_resolution_width", 0);
-	dynresh = remmina_plugin_service->file_get_int(remminafile, "dynamic_resolution_height", 0);
 
-	if (rfi->scale == REMMINA_PROTOCOL_WIDGET_SCALE_MODE_DYNRES && dynresh != 0 && dynresw != 0) {
-		rfi->settings->DesktopWidth = dynresw;
-		rfi->settings->DesktopHeight = dynresh;
+	/* Workaround for FreeRDP issue #5417: in GFX avc modes we can't go under
+	 * AVC_MIN_DESKTOP_WIDTH x AVC_MIN_DESKTOP_HEIGHT */
+	if (rfi->settings->SupportGraphicsPipeline && rfi->settings->GfxH264) {
+		if (rfi->settings->DesktopWidth < AVC_MIN_DESKTOP_WIDTH)
+			rfi->settings->DesktopWidth = AVC_MIN_DESKTOP_WIDTH;
+		if (rfi->settings->DesktopHeight < AVC_MIN_DESKTOP_HEIGHT)
+			rfi->settings->DesktopHeight = AVC_MIN_DESKTOP_HEIGHT;
 	}
+
+	/* Workaround for FreeRDP issue #5119. This will make our horizontal resolution
+	 * an even value, but it will add a vertical black 1 pixel line on the
+	 * right of the desktop */
+	if ((rfi->settings->DesktopWidth & 1) != 0)
+		rfi->settings->DesktopWidth -= 1;
+
 	remmina_plugin_service->protocol_plugin_set_width(gp, rfi->settings->DesktopWidth);
 	remmina_plugin_service->protocol_plugin_set_height(gp, rfi->settings->DesktopHeight);
 
@@ -753,6 +879,17 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 	rfi->settings->GatewayEnabled = FALSE;
 	s = remmina_plugin_service->file_get_string(remminafile, "gateway_server");
 	if (s) {
+		cs = remmina_plugin_service->file_get_string(remminafile, "gwtransp");
+		if (g_strcmp0(cs, "http") == 0) {
+			rfi->settings->GatewayRpcTransport = False;
+			rfi->settings->GatewayHttpTransport = True;
+		}else if (g_strcmp0(cs, "rpc") == 0) {
+			rfi->settings->GatewayRpcTransport = True;
+			rfi->settings->GatewayHttpTransport = False;
+		}else if (g_strcmp0(cs, "auto") == 0) {
+			rfi->settings->GatewayRpcTransport = True;
+			rfi->settings->GatewayHttpTransport = True;
+		}
 		remmina_plugin_service->get_server_port(s, 443, &gateway_host, &gateway_port);
 		rfi->settings->GatewayHostname = gateway_host;
 		rfi->settings->GatewayPort = gateway_port;
@@ -799,6 +936,10 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 	/* Certificate ignore */
 	rfi->settings->IgnoreCertificate = remmina_plugin_service->file_get_int(remminafile, "cert_ignore", 0);
 
+	rfi->settings->AllowUnanouncedOrdersFromServer = remmina_plugin_service->file_get_int(remminafile, "relax-order-checks", 0);
+
+	rfi->settings->GlyphSupportLevel = ( remmina_plugin_service->file_get_int(remminafile, "glyph-cache", 0) ? GLYPH_SUPPORT_FULL : GLYPH_SUPPORT_NONE );
+
 	/* ClientHostname is internally preallocated to 32 bytes by libfreerdp */
 	if ((cs = remmina_plugin_service->file_get_string(remminafile, "clientname"))) {
 		strncpy(rfi->settings->ClientHostname, cs, FREERDP_CLIENTHOSTNAME_LEN - 1);
@@ -849,7 +990,7 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 	g_free(value);
 
 	/* PerformanceFlags bitmask need also to be splitted into BOOL variables
-	 * like rfi->settings->DisableWallpaper, rfi->settings->AllowFontSmoothing...
+	 * like rfi->settings->DisableWallpaper, rfi->settings->AllowFontSmoothing…
 	 * or freerdp_get_param_bool() function will return the wrong value
 	 */
 	freerdp_performance_flags_split(rfi->settings);
@@ -861,9 +1002,6 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 	}
 
 	cs = remmina_plugin_service->file_get_string(remminafile, "security");
-
-
-
 	if (g_strcmp0(cs, "rdp") == 0) {
 		rfi->settings->RdpSecurity = True;
 		rfi->settings->TlsSecurity = False;
@@ -908,12 +1046,17 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 	 * the "disp" dynamic channel, if available */
 	rfi->settings->SupportDisplayControl = TRUE;
 
+	/* Sound settings */
+
 	cs = remmina_plugin_service->file_get_string(remminafile, "sound");
 
 	if (g_strcmp0(cs, "remote") == 0) {
-		rfi->settings->RemoteConsoleAudio = 1;
+		rfi->settings->RemoteConsoleAudio = TRUE;
 	}else if (g_str_has_prefix(cs, "local")) {
 
+		rfi->settings->AudioPlayback = TRUE;
+		rfi->settings->DeviceRedirection = TRUE;	/* rdpsnd requires rdpdr to be registered */
+
 		rdpsnd_nparams = 0;
 		rdpsnd_params[rdpsnd_nparams++] = "rdpsnd";
 
@@ -936,6 +1079,10 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 
 		freerdp_client_add_static_channel(rfi->settings, rdpsnd_nparams, (char**)rdpsnd_params);
 
+	}else {
+		/* Disable sound */
+		rfi->settings->AudioPlayback = FALSE;
+		rfi->settings->RemoteConsoleAudio = FALSE;
 	}
 
 	if ( remmina_plugin_service->file_get_int(remminafile, "microphone", FALSE) ? TRUE : FALSE ) {
@@ -959,6 +1106,9 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 		drive = (RDPDR_DRIVE*)malloc(sizeof(RDPDR_DRIVE));
 		ZeroMemory(drive, sizeof(RDPDR_DRIVE));
 
+		rfi->settings->DeviceRedirection = TRUE;
+		remmina_rdp_load_static_channel_addin(channels, rfi->settings, "rdpdr", rfi->settings);
+
 		s = strrchr( cs, '/' );
 		if ( s == NULL || s[1] == 0 )
 			s = remmina_rdp_plugin_default_drive_name;
@@ -976,15 +1126,24 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 	}
 
 	if (remmina_plugin_service->file_get_int(remminafile, "shareprinter", FALSE)) {
-		RDPDR_PRINTER* printer;
-		printer = (RDPDR_PRINTER*)malloc(sizeof(RDPDR_PRINTER));
-		ZeroMemory(printer, sizeof(RDPDR_PRINTER));
-
-		printer->Type = RDPDR_DTYP_PRINT;
 
 		rfi->settings->DeviceRedirection = TRUE;
+		remmina_rdp_load_static_channel_addin(channels, rfi->settings, "rdpdr", rfi->settings);
+
+		RDPDR_PRINTER* printer;
+		printer = (RDPDR_PRINTER*) calloc(1, sizeof(RDPDR_PRINTER));
+		printer->Type = RDPDR_DTYP_PRINT;
 		rfi->settings->RedirectPrinters = TRUE;
 
+		const gchar* pn = remmina_plugin_service->file_get_string(remminafile, "printername");
+		if ( pn != NULL && pn[0] != '\0' ) {
+			printer->Name = _strdup(pn);
+		}
+		const gchar* dn = remmina_plugin_service->file_get_string(remminafile, "printerdriver");
+		if ( dn != NULL && dn[0] != '\0' ) {
+			printer->DriverName = _strdup(dn);
+		}
+
 		freerdp_device_collection_add(rfi->settings, (RDPDR_DEVICE*)printer);
 	}
 
@@ -995,14 +1154,85 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 
 		smartcard->Type = RDPDR_DTYP_SMARTCARD;
 
-		smartcard->Name = _strdup("scard");
-
 		rfi->settings->DeviceRedirection = TRUE;
+		remmina_rdp_load_static_channel_addin(channels, rfi->settings, "rdpdr", rfi->settings);
+
+		const gchar* sn = remmina_plugin_service->file_get_string(remminafile, "smartcardname");
+		if ( sn != NULL && sn[0] != '\0' ) {
+			smartcard->Name = _strdup(sn);
+		}
+
 		rfi->settings->RedirectSmartCards = TRUE;
 
 		freerdp_device_collection_add(rfi->settings, (RDPDR_DEVICE*)smartcard);
 	}
 
+	if (remmina_plugin_service->file_get_int(remminafile, "passwordispin", FALSE)) {
+		/* Option works only combined with Username and Domain, because freerdp
+		 * doesn’t know anything about information on smartcard */
+		rfi->settings->PasswordIsSmartcardPin = TRUE;
+	}
+
+	/* /serial[:<name>[,<path>[,<driver>[,permissive]]]] */
+	if (remmina_plugin_service->file_get_int(remminafile, "shareserial", FALSE)) {
+		RDPDR_SERIAL* serial;
+		serial = (RDPDR_SERIAL*)malloc(sizeof(RDPDR_SERIAL));
+		ZeroMemory(serial, sizeof(RDPDR_SERIAL));
+
+		serial->Type = RDPDR_DTYP_SERIAL;
+
+		rfi->settings->DeviceRedirection = TRUE;
+		remmina_rdp_load_static_channel_addin(channels, rfi->settings, "rdpdr", rfi->settings);
+
+		const gchar* sn = remmina_plugin_service->file_get_string(remminafile, "serialname");
+		if ( sn != NULL && sn[0] != '\0' ) {
+			serial->Name = _strdup(sn);
+		}
+
+		const gchar* sd = remmina_plugin_service->file_get_string(remminafile, "serialdriver");
+		if ( sd != NULL && sd[0] != '\0' ) {
+			serial->Driver = _strdup(sd);
+		}
+
+		const gchar* sp = remmina_plugin_service->file_get_string(remminafile, "serialpath");
+		if ( sp != NULL && sp[0] != '\0' ) {
+			serial->Path = _strdup(sp);
+		}
+
+		if (remmina_plugin_service->file_get_int(remminafile, "serialpermissive", FALSE)) {
+			serial->Permissive = _strdup("permissive");
+		}
+
+		rfi->settings->RedirectSerialPorts = TRUE;
+
+		freerdp_device_collection_add(rfi->settings, (RDPDR_DEVICE*)serial);
+	}
+
+	if (remmina_plugin_service->file_get_int(remminafile, "shareparallel", FALSE)) {
+		RDPDR_PARALLEL* parallel;
+		parallel = (RDPDR_PARALLEL*)malloc(sizeof(RDPDR_PARALLEL));
+		ZeroMemory(parallel, sizeof(RDPDR_PARALLEL));
+
+		parallel->Type = RDPDR_DTYP_PARALLEL;
+
+		rfi->settings->DeviceRedirection = TRUE;
+		remmina_rdp_load_static_channel_addin(channels, rfi->settings, "rdpdr", rfi->settings);
+
+		rfi->settings->RedirectParallelPorts = TRUE;
+
+		const gchar* pn = remmina_plugin_service->file_get_string(remminafile, "parallelname");
+		if ( pn != NULL && pn[0] != '\0' ) {
+			parallel->Name = _strdup(pn);
+		}
+		const gchar* dp = remmina_plugin_service->file_get_string(remminafile, "parallelpath");
+		if ( dp != NULL && dp[0] != '\0' ) {
+			parallel->Path = _strdup(dp);
+		}
+
+		freerdp_device_collection_add(rfi->settings, (RDPDR_DEVICE*)parallel);
+	}
+
+
 	if (!freerdp_connect(rfi->instance)) {
 		if (!rfi->user_cancelled) {
 			UINT32 e;
@@ -1018,7 +1248,8 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 				remmina_plugin_service->protocol_plugin_set_error(gp, _("Authentication to RDP server %s failed.\nCheck username, password and domain."),
 					rfi->settings->ServerHostname );
 				// Invalidate the saved password, so the user will be re-asked at next logon
-				remmina_plugin_service->file_unsave_password(remminafile);
+				if (!remmina_rdp_authenticate(rfi->instance, NULL, NULL, NULL))
+					remmina_plugin_service->file_unsave_password(remminafile);
 				break;
 			case STATUS_ACCOUNT_LOCKED_OUT:
 #ifdef FREERDP_ERROR_CONNECT_ACCOUNT_LOCKED_OUT
@@ -1062,6 +1293,15 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 				remmina_plugin_service->protocol_plugin_set_error(gp, _("Access to RDP server %s failed.\nAccount has restrictions."),
 					rfi->settings->ServerHostname );
 				break;
+
+			case STATUS_PASSWORD_MUST_CHANGE:
+#ifdef FREERDP_ERROR_CONNECT_PASSWORD_MUST_CHANGE
+			case FREERDP_ERROR_CONNECT_PASSWORD_MUST_CHANGE:
+#endif
+			remmina_plugin_service->protocol_plugin_set_error(gp, _("Access to RDP server %s failed.\nUser must change password before connecting."),
+				rfi->settings->ServerHostname );
+			break;
+
 			case FREERDP_ERROR_CONNECT_FAILED:
 				remmina_plugin_service->protocol_plugin_set_error(gp, _("Connection to RDP server %s failed."), rfi->settings->ServerHostname );
 				break;
@@ -1087,13 +1327,18 @@ static gboolean remmina_rdp_main(RemminaProtocolWidget* gp)
 						remmina_plugin_service->protocol_plugin_set_error(gp, _("Unable to initialize libfreerdp gdi") );
 						break;
 					case REMMINA_POSTCONNECT_ERROR_NO_H264:
-						remmina_plugin_service->protocol_plugin_set_error(gp, _("You requested an H264 GFX mode for server %s, but your libfreerdp does not support H264. Please use a non-AVC Color Depth setting."), rfi->settings->ServerHostname);
+						remmina_plugin_service->protocol_plugin_set_error(gp, _("You requested an H.264 GFX mode for server %s, but your libfreerdp does not support H.264. Please use a non-AVC Color Depth setting."), rfi->settings->ServerHostname);
 						break;
 				}
 				break;
+#endif
+#ifdef FREERDP_ERROR_SERVER_DENIED_CONNECTION
+			case FREERDP_ERROR_SERVER_DENIED_CONNECTION:
+				remmina_plugin_service->protocol_plugin_set_error(gp, _("Server %s denied the connection."), rfi->settings->ServerHostname );
+				break;
 #endif
 			default:
-				g_printf("%08X %08X\n", e, (unsigned)ERRCONNECT_POST_CONNECT_FAILED);
+				g_printf("libfreerdp returned code is %08X\n", e);
 				remmina_plugin_service->protocol_plugin_set_error(gp, _("Unable to connect to RDP server %s"), rfi->settings->ServerHostname);
 				break;
 			}
@@ -1140,6 +1385,7 @@ static void remmina_rdp_init(RemminaProtocolWidget* gp)
 	instance->PreConnect = remmina_rdp_pre_connect;
 	instance->PostConnect = remmina_rdp_post_connect;
 	instance->Authenticate = remmina_rdp_authenticate;
+	instance->GatewayAuthenticate = remmina_rdp_gw_authenticate;
 	instance->VerifyCertificate = remmina_rdp_verify_certificate;
 	instance->VerifyChangedCertificate = remmina_rdp_verify_changed_certificate;
 
@@ -1169,7 +1415,7 @@ static gboolean remmina_rdp_open_connection(RemminaProtocolWidget* gp)
 
 	if (pthread_create(&rfi->thread, NULL, remmina_rdp_main_thread, gp)) {
 		remmina_plugin_service->protocol_plugin_set_error(gp, "%s",
-			"Failed to initialize pthread. Falling back to non-thread mode...");
+			"Failed to initialize pthread. Falling back to non-thread mode…");
 
 		rfi->thread = 0;
 
@@ -1263,17 +1509,25 @@ static void remmina_rdp_call_feature(RemminaProtocolWidget* gp, const RemminaPro
 		break;
 
 	case REMMINA_RDP_FEATURE_SCALE:
-		rfi->scale = remmina_plugin_service->remmina_protocol_widget_get_current_scale_mode(gp);
-		remmina_rdp_event_update_scale(gp);
+		if (rfi) {
+			rfi->scale = remmina_plugin_service->remmina_protocol_widget_get_current_scale_mode(gp);
+			remmina_rdp_event_update_scale(gp);
+		} else {
+			printf("REMMINA RDP PLUGIN WARNING: rfi is null in %s REMMINA_RDP_FEATURE_SCALE\n", __func__);
+		}
 		break;
 
 	case REMMINA_RDP_FEATURE_DYNRESUPDATE:
 		break;
 
 	case REMMINA_RDP_FEATURE_TOOL_REFRESH:
-		gtk_widget_queue_draw_area(rfi->drawing_area, 0, 0,
-			remmina_plugin_service->protocol_plugin_get_width(gp),
-			remmina_plugin_service->protocol_plugin_get_height(gp));
+		if (rfi) {
+			gtk_widget_queue_draw_area(rfi->drawing_area, 0, 0,
+				remmina_plugin_service->protocol_plugin_get_width(gp),
+				remmina_plugin_service->protocol_plugin_get_height(gp));
+		} else {
+			printf("REMMINA RDP PLUGIN WARNING: rfi is null in %s REMMINA_RDP_FEATURE_TOOL_REFRESH\n", __func__);
+		}
 		break;
 
 	case REMMINA_RDP_FEATURE_TOOL_SENDCTRLALTDEL:
@@ -1313,7 +1567,7 @@ static gboolean remmina_rdp_get_screenshot(RemminaProtocolWidget *gp, RemminaPlu
 	bitsPerPixel = GetBitsPerPixel(gdi->hdc->format);
 
 	/** @todo we should lock freerdp subthread to update rfi->primary_buffer, rfi->gdi and w/h,
-	 * from here to memcpy, but... how ? */
+	 * from here to memcpy, but… how ? */
 
 	szmem = gdi->width * gdi->height * bytesPerPixel;
 
@@ -1383,6 +1637,14 @@ static gpointer security_list[] =
 	NULL
 };
 
+static gpointer gwtransp_list[] =
+{
+	"http", "http",
+	"rpc",  "rpc",
+	"auto", "auto",
+	NULL
+};
+
 /* Array of RemminaProtocolSetting for basic settings.
  * Each item is composed by:
  * a) RemminaProtocolSettingType for setting type
@@ -1401,7 +1663,6 @@ static const RemminaProtocolSetting remmina_rdp_basic_settings[] =
 	{ REMMINA_PROTOCOL_SETTING_TYPE_RESOLUTION, "resolution",		NULL,					    FALSE,	NULL,		    NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	    "colordepth",		N_("Color depth"),			    FALSE,	colordepth_list,    NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_FOLDER,	    "sharefolder",		N_("Share folder"),			    FALSE,	NULL,		    NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "disableautoreconnect",	N_("Disable automatic reconnection"),	    FALSE,	NULL,		    NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	    NULL,			NULL,					    FALSE,	NULL,		    NULL}
 };
 
@@ -1419,6 +1680,7 @@ static const RemminaProtocolSetting remmina_rdp_advanced_settings[] =
 	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	    "quality",		       N_("Quality"),				FALSE,	quality_list,	NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	    "sound",		       N_("Sound"),				FALSE,	sound_list,	NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	    "security",		       N_("Security"),				FALSE,	security_list,	NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	    "gwtransp",		       N_("Gateway transport type"),		FALSE,	gwtransp_list,	NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "gateway_server",	       N_("RD Gateway server"),			FALSE,	NULL,		NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "gateway_username",	       N_("RD Gateway username"),		FALSE,	NULL,		NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD,   "gateway_password",	       N_("RD Gateway password"),		FALSE,	NULL,		NULL},
@@ -1426,16 +1688,33 @@ static const RemminaProtocolSetting remmina_rdp_advanced_settings[] =
 	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "clientname",	       N_("Client name"),			FALSE,	NULL,		NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "exec",		       N_("Startup program"),			FALSE,	NULL,		NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "execpath",		       N_("Startup path"),			FALSE,	NULL,		NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "loadbalanceinfo",	       N_("Load Balance Info"),			FALSE,	NULL,		NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "cert_ignore",	       N_("Ignore certificate"),		TRUE,	NULL,		NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "microphone",	       N_("Redirect local microphone"),		TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "loadbalanceinfo",	       N_("Load balance info"),			FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "printername",	       N_("Local printer name"),		FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "printerdriver",	       N_("Local printer driver"),		FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "serialname",	       N_("Local serial name"),			FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "serialdriver",	       N_("Local serial driver"),		FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "serialpath",	       N_("Local serial path"),			FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "parallelname",	       N_("Local parallel name"),		FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "parallelpath",	       N_("Local parallel device"),		FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	    "smartcardname",	       N_("Smartcard Name"),			FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "shareprinter",	       N_("Share printers"),			TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "shareserial",	       N_("Share serial ports"),		TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "serialpermissive",	       N_("Serial ports permissive mode"),	TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "shareparallel",	       N_("Share parallel ports"),		TRUE,	NULL,		NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "sharesmartcard",	       N_("Share smartcard"),			TRUE,	NULL,		NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "shareprinter",	       N_("Share local printers"),		TRUE,	NULL,		NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "disablepasswordstoring",  N_("Disable password storing"),		TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "microphone",	       N_("Redirect local microphone"),		TRUE,	NULL,		NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "disableclipboard",	       N_("Disable clipboard sync"),		TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "cert_ignore",	       N_("Ignore certificate"),		TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "disablepasswordstoring",  N_("Disable password storing"),		TRUE,	NULL,		NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "console",		       N_("Attach to console (2003/2003 R2)"),	TRUE,	NULL,		NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "disable_fastpath",	       N_("Disable fast-path"),	TRUE,	NULL,		NULL},
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "gateway_usage",	       N_("Server detection using RD Gateway"),	FALSE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "disable_fastpath",	       N_("Disable fast-path"),			TRUE,	NULL,		NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "gateway_usage",	       N_("Server detection using RD Gateway"),	TRUE,	NULL,		NULL},
+#if defined(PROXY_TYPE_IGNORE)
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "enableproxy",	       N_("Enable proxy support"),		TRUE,	NULL,		NULL},
+#endif
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "disableautoreconnect",    N_("Disable automatic reconnection"),	TRUE,	NULL,	        NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "relax-order-checks",      N_("Relax Order Checks"),	        TRUE,	NULL,	        NULL},
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	    "glyph-cache",             N_("Glyph Cache"),	                TRUE,	NULL,	        NULL},
 	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	    NULL,			NULL,					FALSE,	NULL,		NULL}
 };
 
@@ -1462,8 +1741,8 @@ static RemminaProtocolPlugin remmina_rdp =
 	N_("RDP - Remote Desktop Protocol"),            // Description
 	GETTEXT_PACKAGE,                                // Translation domain
 	remmina_plugin_rdp_version,                     // Version number
-	"remmina-rdp",                                  // Icon for normal connection
-	"remmina-rdp-ssh",                              // Icon for SSH connection
+	"remmina-rdp-symbolic",                         // Icon for normal connection
+	"remmina-rdp-ssh-symbolic",                     // Icon for SSH connection
 	remmina_rdp_basic_settings,                     // Array for basic settings
 	remmina_rdp_advanced_settings,                  // Array for advanced settings
 	REMMINA_PROTOCOL_SSH_SETTING_TUNNEL,            // SSH settings type
@@ -1580,7 +1859,7 @@ G_MODULE_EXPORT gboolean remmina_plugin_entry(RemminaPluginService* service)
 	}
 
 	snprintf(remmina_plugin_rdp_version, sizeof(remmina_plugin_rdp_version),
-		"RDP Plugin: %s (git %s), Compiled with FreeRDP lib: %s (%s), Running with FreeRDP lib: %s (rev %s), H264: %s",
+		"RDP Plugin: %s (git %s), Compiled with FreeRDP lib: %s (%s), Running with FreeRDP lib: %s (rev %s), H.264: %s",
 		VERSION, REMMINA_GIT_REVISION,
 		FREERDP_VERSION_FULL, GIT_REVISION,
 		freerdp_get_version_string(),
