@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2010 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2018 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,8 @@
 #include <stdlib.h>
 
 #include "config.h"
+#include "remmina_sodium.h"
+#include "remmina.h"
 #include "remmina_exec.h"
 #include "remmina_file_manager.h"
 #include "remmina_icon.h"
@@ -53,6 +55,7 @@
 #include "remmina_ssh_plugin.h"
 #include "remmina_widget_pool.h"
 #include "remmina/remmina_trace_calls.h"
+#include "rmnews.h"
 #include "remmina_stats_sender.h"
 
 
@@ -73,13 +76,16 @@ static int gcrypt_thread_initialized = 0;
 #endif  /* !GCRYPT_VERSION_NUMBER */
 #endif  /* HAVE_LIBGCRYPT */
 
+gboolean kioskmode;
+
 static GOptionEntry remmina_options[] =
 {
 	{ "about",	  'a', 0,		     G_OPTION_ARG_NONE,		    NULL, N_("Show about dialog"),					       NULL	  },
-	{ "connect",	  'c', 0,		     G_OPTION_ARG_FILENAME,	    NULL, N_("Connect to a .remmina file"),				       "FILE"	  },
-	{ G_OPTION_REMAINING, '\0', 0,		     G_OPTION_ARG_FILENAME_ARRAY,   NULL, N_("Connect to a .remmina file"),				       "FILE"	  },
-	{ "edit",	  'e', 0,		     G_OPTION_ARG_FILENAME,	    NULL, N_("Edit a .remmina file"),					       "FILE"	  },
+	{ "connect",	  'c', 0,		     G_OPTION_ARG_FILENAME,	    NULL, N_("Connect to desktop described by file (.remmina or type supported by plugin)"),				       "FILE"	  },
+	{ G_OPTION_REMAINING, '\0', 0,		     G_OPTION_ARG_FILENAME_ARRAY,   NULL, N_("Connect to desktop described by file (.remmina or type supported by plugin)"),				       "FILE"	  },
+	{ "edit",	  'e', 0,		     G_OPTION_ARG_FILENAME,	    NULL, N_("Edit desktop connection described by file (.remmina or type supported by plugin)"),					       "FILE"	  },
 	{ "help",	  '?', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE,	   	    NULL, NULL,								       NULL	  },
+	{ "kiosk",	  'k', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Start Remmina in Kiosk mode"),				       NULL	  },
 	{ "new",	  'n', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Create a new connection profile"),			       NULL	  },
 	{ "pref",	  'p', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Show preferences dialog page"),				       "PAGENR"	  },
 	{ "plugin",	  'x', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Execute the plugin"),					       "PLUGIN"	  },
@@ -87,8 +93,8 @@ static GOptionEntry remmina_options[] =
 	{ "server",	  's', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Use default server name (for --new)"),			       "SERVER"	  },
 	{ "protocol",	  't', 0,		     G_OPTION_ARG_STRING,  	    NULL, N_("Use default protocol (for --new)"),			       "PROTOCOL" },
 	{ "icon",	  'i', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Start as tray icon"),					       NULL	  },
-	{ "version",	  'v', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Show the application's version"),				       NULL	  },
-	{ "full-version", 'V', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Show the application's version, including the pulgin versions"), NULL	  },
+	{ "version",	  'v', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Show the application’s version"),				       NULL	  },
+	{ "full-version", 'V', 0,		     G_OPTION_ARG_NONE,	   	    NULL, N_("Show the application’s version, including the plugin versions"), NULL	  },
 	{ NULL }
 };
 
@@ -116,6 +122,11 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 	gchar *protocol;
 	gchar *server;
 
+#if SODIUM_VERSION_INT >= 90200
+	remmina_sodium_init();
+#endif
+	remmina_pref_init();
+
 	opts = g_application_command_line_get_options_dict(cmdline);
 
 	if (g_variant_dict_lookup_value(opts, "quit", NULL)) {
@@ -131,7 +142,7 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 
 	/** @todo This should be a G_OPTION_ARG_FILENAME_ARRAY (^aay) so that
 	 * we can implement multi profile connection:
-	 *    https://github.com/FreeRDP/Remmina/issues/915
+	 *    https://gitlab.com/Remmina/Remmina/issues/915
 	 */
 	if (g_variant_dict_lookup(opts, "connect", "^ay", &str)) {
 		remmina_exec_command(REMMINA_COMMAND_CONNECT, g_strdup(str));
@@ -151,6 +162,12 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 		executed = TRUE;
 	}
 
+	if (g_variant_dict_lookup_value(opts, "kiosk", NULL)) {
+		kioskmode = TRUE;
+		remmina_exec_command(REMMINA_COMMAND_MAIN, NULL);
+		executed = TRUE;
+	}
+
 	if (g_variant_dict_lookup_value(opts, "new", NULL)) {
 		if (!g_variant_dict_lookup(opts, "protocol", "&s", &protocol))
 			protocol = NULL;
@@ -204,8 +221,8 @@ static void remmina_on_startup(GApplication *app)
 
 	RemminaSecretPlugin *secret_plugin;
 
-	remmina_file_manager_init();
 	remmina_pref_init();
+	remmina_file_manager_init();
 	remmina_plugin_manager_init();
 	remmina_widget_pool_init();
 	remmina_sftp_plugin_register();
@@ -215,11 +232,16 @@ static void remmina_on_startup(GApplication *app)
 	g_set_application_name("Remmina");
 	gtk_window_set_default_icon_name(REMMINA_APP_ID);
 
+	/* Setting the X11 program class (WM_CLASS) is necessary to group
+	 * windows with .desktop file which has the same StartupWMClass */
+	gdk_set_program_class(REMMINA_APP_ID);
+
 	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(),
 		REMMINA_RUNTIME_DATADIR G_DIR_SEPARATOR_S "icons");
 	g_application_hold(app);
 
 	remmina_stats_sender_schedule();
+	rmnews_schedule();
 
 	/* Check for secret plugin and service initialization and show some warnings on the console if
 	 * there is something missing */
@@ -253,7 +275,12 @@ int main(int argc, char* argv[])
 	const gchar *app_id;
 	int status;
 
-	gdk_set_allowed_backends("x11,broadway,quartz,mir");
+	/* Enable wayland backend only after GTK 3.22.27 or the clipboard
+	 * will not work. See GTK bug 790031 */
+	if (remmina_gtk_check_version(3,22,27))
+		gdk_set_allowed_backends("wayland,x11,broadway,quartz,mir");
+	else
+		gdk_set_allowed_backends("x11,broadway,quartz,mir");
 
 	remmina_masterthread_exec_save_main_thread_id();
 
@@ -278,6 +305,9 @@ int main(int argc, char* argv[])
 
 	app_id = g_application_id_is_valid(REMMINA_APP_ID) ? REMMINA_APP_ID : NULL;
 	app = gtk_application_new(app_id, G_APPLICATION_HANDLES_COMMAND_LINE);
+#if !GTK_CHECK_VERSION(4, 0, 0) /* This is not needed anymore starting from Gtk+ 4 */
+	g_set_prgname(app_id);
+#endif
 	g_signal_connect(app, "startup", G_CALLBACK(remmina_on_startup), NULL);
 	g_signal_connect(app, "command-line", G_CALLBACK(remmina_on_command_line), NULL);
 	g_signal_connect(app, "handle-local-options", G_CALLBACK(remmina_on_local_cmdline), NULL);
