@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2018 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#include "remmina_sodium.h"
+
 #include "remmina_public.h"
 #include "remmina_string_array.h"
 #include "remmina_pref.h"
@@ -169,7 +171,7 @@ static void remmina_pref_init_keymap(void)
 static gboolean remmina_pref_file_do_copy(const char *src_path, const char *dst_path)
 {
 	GFile *src = g_file_new_for_path(src_path), *dst = g_file_new_for_path(dst_path);
-	/* We don't overwrite the target if it exists, because overwrite is not set */
+	/* We don’t overwrite the target if it exists, because overwrite is not set */
 	const gboolean ok = g_file_copy(src, dst, G_FILE_COPY_NONE, NULL, NULL, NULL, NULL);
 
 	g_object_unref(dst);
@@ -221,14 +223,14 @@ void remmina_pref_init(void)
 	TRACE_CALL(__func__);
 	GKeyFile *gkeyfile;
 	gchar *remmina_dir;
-	const gchar *filename = g_strdup_printf("%s.pref", g_get_prgname());
-	const gchar *colors_filename = g_strdup_printf("%s.colors", g_get_prgname());
+	const gchar *filename = "remmina.pref";
+	const gchar *colors_filename = "remmina.colors";
 	gchar *remmina_colors_file;
 	GDir *dir;
-	gchar *legacy = g_strdup_printf(".%s", g_get_prgname());
+	gchar *legacy = ".remmina";
 	int i;
 
-	remmina_dir = g_build_path( "/", g_get_user_config_dir(), g_get_prgname(), NULL);
+	remmina_dir = g_build_path( "/", g_get_user_config_dir(), "remmina", NULL);
 	/* Create the XDG_CONFIG_HOME directory */
 	g_mkdir_with_parents(remmina_dir, 0750);
 
@@ -241,21 +243,21 @@ void remmina_pref_init(void)
 		remmina_pref_file_do_copy(
 			g_build_path( "/", remmina_dir, filename, NULL),
 			g_build_path( "/", g_get_user_config_dir(),
-				g_get_prgname(), filename, NULL));
+				"remmina", filename, NULL));
 	}
 
 	/* /usr/local/etc/remmina */
 	const gchar * const *dirs = g_get_system_config_dirs();
 	g_free(remmina_dir), remmina_dir = NULL;
 	for (i = 0; dirs[i] != NULL; ++i) {
-		remmina_dir = g_build_path( "/", dirs[i], g_get_prgname(), NULL);
+		remmina_dir = g_build_path( "/", dirs[i], "remmina", NULL);
 		if (g_file_test(remmina_dir, G_FILE_TEST_IS_DIR)) {
 			dir = g_dir_open(remmina_dir, 0, NULL);
 			while ((filename = g_dir_read_name(dir)) != NULL) {
 				remmina_pref_file_do_copy(
 					g_build_path( "/", remmina_dir, filename, NULL),
 					g_build_path( "/", g_get_user_config_dir(),
-						g_get_prgname(), filename, NULL));
+						"remmina", filename, NULL));
 			}
 			g_free(remmina_dir), remmina_dir = NULL;
 		}
@@ -265,7 +267,7 @@ void remmina_pref_init(void)
 	if (remmina_dir != NULL)
 		g_free(remmina_dir), remmina_dir = NULL;
 	remmina_dir = g_build_path( "/", g_get_user_config_dir(),
-		g_get_prgname(), NULL);
+		"remmina", NULL);
 
 	remmina_pref_file = g_strdup_printf("%s/remmina.pref", remmina_dir);
 	/* remmina.colors */
@@ -281,6 +283,22 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.save_view_mode = TRUE;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "use_master_password", NULL))
+		remmina_pref.use_master_password = g_key_file_get_boolean(gkeyfile, "remmina_pref", "use_master_password", NULL);
+	else
+		remmina_pref.use_master_password = FALSE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "unlock_timeout", NULL))
+		remmina_pref.unlock_timeout  = g_key_file_get_integer(gkeyfile, "remmina_pref", "unlock_timeout", NULL);
+	else
+		remmina_pref.unlock_timeout  = 0;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "unlock_password", NULL))
+		remmina_pref.unlock_password  = g_key_file_get_string(gkeyfile, "remmina_pref", "unlock_password", NULL);
+	else
+		remmina_pref.unlock_password = g_strdup("");
+
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "fullscreen_on_auto", NULL))
 		remmina_pref.fullscreen_on_auto = g_key_file_get_boolean(gkeyfile, "remmina_pref", "fullscreen_on_auto", NULL);
 	else
@@ -296,6 +314,11 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.prevent_snap_welcome_message = FALSE;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "last_quickconnect_protocol", NULL))
+		remmina_pref.last_quickconnect_protocol = g_key_file_get_string(gkeyfile, "remmina_pref", "last_quickconnect_protocol", NULL);
+	else
+		remmina_pref.last_quickconnect_protocol = g_strdup("");
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "toolbar_placement", NULL))
 		remmina_pref.toolbar_placement = g_key_file_get_integer(gkeyfile, "remmina_pref", "toolbar_placement", NULL);
 	else
@@ -312,6 +335,12 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.hide_connection_toolbar = FALSE;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "hide_searchbar", NULL))
+		remmina_pref.hide_searchbar = g_key_file_get_boolean(gkeyfile, "remmina_pref",
+			"hide_searchbar", NULL);
+	else
+		remmina_pref.hide_searchbar = FALSE;
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "default_action", NULL))
 		remmina_pref.default_action = g_key_file_get_integer(gkeyfile, "remmina_pref", "default_action", NULL);
 	else
@@ -327,11 +356,6 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.hide_toolbar = FALSE;
 
-	if (g_key_file_has_key(gkeyfile, "remmina_pref", "hide_statusbar", NULL))
-		remmina_pref.hide_statusbar = g_key_file_get_boolean(gkeyfile, "remmina_pref", "hide_statusbar", NULL);
-	else
-		remmina_pref.hide_statusbar = FALSE;
-
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "small_toolbutton", NULL))
 		remmina_pref.small_toolbutton = g_key_file_get_boolean(gkeyfile, "remmina_pref", "small_toolbutton", NULL);
 	else
@@ -393,6 +417,22 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.ssh_loglevel = DEFAULT_SSH_LOGLEVEL;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "deny_screenshot_clipboard", NULL))
+		remmina_pref.deny_screenshot_clipboard = g_key_file_get_boolean(gkeyfile, "remmina_pref", "deny_screenshot_clipboard", NULL);
+	else
+		remmina_pref.deny_screenshot_clipboard = TRUE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "datadir_path", NULL))
+		remmina_pref.datadir_path = g_key_file_get_string(gkeyfile, "remmina_pref", "datadir_path", NULL);
+	else
+		remmina_pref.datadir_path = g_strdup("");
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "remmina_file_name", NULL)) {
+		remmina_pref.remmina_file_name = g_key_file_get_string(gkeyfile, "remmina_pref", "remmina_file_name", NULL);
+	}else{
+		remmina_pref.remmina_file_name = g_strdup("%G_%P_%N_%h");
+	}
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "screenshot_path", NULL)) {
 		remmina_pref.screenshot_path = g_key_file_get_string(gkeyfile, "remmina_pref", "screenshot_path", NULL);
 	}else{
@@ -401,6 +441,12 @@ void remmina_pref_init(void)
 			remmina_pref.screenshot_path = g_get_home_dir();
 	}
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "screenshot_name", NULL)) {
+		remmina_pref.screenshot_name = g_key_file_get_string(gkeyfile, "remmina_pref", "screenshot_name", NULL);
+	}else{
+		remmina_pref.screenshot_name = g_strdup("remmina_%p_%h_%Y%m%d-%H%M%S");
+	}
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "ssh_parseconfig", NULL))
 		remmina_pref.ssh_parseconfig = g_key_file_get_boolean(gkeyfile, "remmina_pref", "ssh_parseconfig", NULL);
 	else
@@ -476,22 +522,6 @@ void remmina_pref_init(void)
 		remmina_pref.fullscreen_toolbar_visibility = g_key_file_get_integer(gkeyfile, "remmina_pref", "fullscreen_toolbar_visibility", NULL);
 	else
 		remmina_pref.fullscreen_toolbar_visibility = FLOATING_TOOLBAR_VISIBILITY_PEEKING;
-	/* Show buttons icons */
-	if (g_key_file_has_key(gkeyfile, "remmina_pref", "show_buttons_icons", NULL)) {
-		remmina_pref.show_buttons_icons = g_key_file_get_integer(gkeyfile, "remmina_pref", "show_buttons_icons", NULL);
-		if (remmina_pref.show_buttons_icons) {
-			g_object_set(gtk_settings_get_default(), "gtk-button-images", remmina_pref.show_buttons_icons == 1, NULL);
-		}
-	}else
-		remmina_pref.show_buttons_icons = 0;
-	/* Show menu icons */
-	if (g_key_file_has_key(gkeyfile, "remmina_pref", "show_menu_icons", NULL)) {
-		remmina_pref.show_menu_icons = g_key_file_get_integer(gkeyfile, "remmina_pref", "show_menu_icons", NULL);
-		if (remmina_pref.show_menu_icons) {
-			g_object_set(gtk_settings_get_default(), "gtk-menu-images", remmina_pref.show_menu_icons == 1, NULL);
-		}
-	}else
-		remmina_pref.show_menu_icons = 0;
 
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "auto_scroll_step", NULL))
 		remmina_pref.auto_scroll_step = g_key_file_get_integer(gkeyfile, "remmina_pref", "auto_scroll_step", NULL);
@@ -618,11 +648,6 @@ void remmina_pref_init(void)
 
 	remmina_pref_file_load_colors(gkeyfile, &remmina_pref.color_pref);
 
-	if (g_key_file_has_key(gkeyfile, "usage_stats", "periodic_usage_stats_permission_asked", NULL))
-		remmina_pref.periodic_usage_stats_permission_asked = g_key_file_get_boolean(gkeyfile, "usage_stats", "periodic_usage_stats_permission_asked", NULL);
-	else
-		remmina_pref.periodic_usage_stats_permission_asked = FALSE;
-
 	if (g_key_file_has_key(gkeyfile, "usage_stats", "periodic_usage_stats_permitted", NULL))
 		remmina_pref.periodic_usage_stats_permitted = g_key_file_get_boolean(gkeyfile, "usage_stats", "periodic_usage_stats_permitted", NULL);
 	else
@@ -638,6 +663,18 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.periodic_usage_stats_uuid_prefix = NULL;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_news", "periodic_rmnews_last_get", NULL))
+		remmina_pref.periodic_rmnews_last_get = g_key_file_get_int64(gkeyfile, "remmina_news", "periodic_rmnews_last_get", NULL);
+	else
+		remmina_pref.periodic_rmnews_last_get = 0;
+
+	/* Default settings */
+	if (!g_key_file_has_key(gkeyfile, "remmina", "name", NULL)) {
+		g_key_file_set_string(gkeyfile, "remmina", "name", "");
+		g_key_file_set_integer(gkeyfile, "remmina", "ignore-tls-errors", 1);
+		g_key_file_set_integer(gkeyfile, "remmina", "enable-plugins", 1);
+		remmina_pref_save();
+	}
 
 	g_key_file_free(gkeyfile);
 
@@ -647,9 +684,24 @@ void remmina_pref_init(void)
 	remmina_pref_init_keymap();
 }
 
+gboolean remmina_pref_is_rw(void)
+{
+	TRACE_CALL(__func__);
+	if (access(remmina_pref_file, W_OK) == 0)
+		return TRUE;
+	else
+		return FALSE;
+	return FALSE;
+}
+
 gboolean remmina_pref_save(void)
 {
 	TRACE_CALL(__func__);
+
+	if (remmina_pref_is_rw() == FALSE) {
+		g_debug ("remmina.pref is not writable, returning");
+		return FALSE;
+	}
 	GKeyFile *gkeyfile;
 	GError *error = NULL;
 	gchar *content;
@@ -659,20 +711,34 @@ gboolean remmina_pref_save(void)
 
 	g_key_file_load_from_file(gkeyfile, remmina_pref_file, G_KEY_FILE_NONE, NULL);
 
+	g_key_file_set_string(gkeyfile, "remmina_pref", "datadir_path", remmina_pref.datadir_path);
+	g_key_file_set_string(gkeyfile, "remmina_pref", "remmina_file_name", remmina_pref.remmina_file_name);
+	g_key_file_set_string(gkeyfile, "remmina_pref", "screenshot_path", remmina_pref.screenshot_path);
+	g_key_file_set_string(gkeyfile, "remmina_pref", "remmina_file_name", remmina_pref.remmina_file_name);
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "deny_screenshot_clipboard", remmina_pref.deny_screenshot_clipboard);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "save_view_mode", remmina_pref.save_view_mode);
+#if SODIUM_VERSION_INT >= 90200
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "use_master_password", remmina_pref.use_master_password);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "unlock_timeout", remmina_pref.unlock_timeout);
+	g_key_file_set_string(gkeyfile, "remmina_pref", "unlock_password", remmina_pref.unlock_password);
+#else
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "use_master_password", FALSE);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "unlock_timeout", 0);
+	g_key_file_set_string(gkeyfile, "remmina_pref", "unlock_password", g_strdup(""));
+#endif
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "floating_toolbar_placement", remmina_pref.floating_toolbar_placement);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "toolbar_placement", remmina_pref.toolbar_placement);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "prevent_snap_welcome_message", remmina_pref.prevent_snap_welcome_message);
+	g_key_file_set_string(gkeyfile, "remmina_pref", "last_quickconnect_protocol", remmina_pref.last_quickconnect_protocol);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "fullscreen_on_auto", remmina_pref.fullscreen_on_auto);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "always_show_tab", remmina_pref.always_show_tab);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "hide_connection_toolbar", remmina_pref.hide_connection_toolbar);
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "hide_searchbar", remmina_pref.hide_searchbar);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "default_action", remmina_pref.default_action);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "scale_quality", remmina_pref.scale_quality);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "ssh_loglevel", remmina_pref.ssh_loglevel);
-	g_key_file_set_string(gkeyfile, "remmina_pref", "screenshot_path", remmina_pref.screenshot_path);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "ssh_parseconfig", remmina_pref.ssh_parseconfig);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "hide_toolbar", remmina_pref.hide_toolbar);
-	g_key_file_set_boolean(gkeyfile, "remmina_pref", "hide_statusbar", remmina_pref.hide_statusbar);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "small_toolbutton", remmina_pref.small_toolbutton);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "view_file_mode", remmina_pref.view_file_mode);
 	g_key_file_set_string(gkeyfile, "remmina_pref", "resolutions", remmina_pref.resolutions);
@@ -698,8 +764,6 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "default_mode", remmina_pref.default_mode);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "tab_mode", remmina_pref.tab_mode);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "fullscreen_toolbar_visibility", remmina_pref.fullscreen_toolbar_visibility);
-	g_key_file_set_integer(gkeyfile, "remmina_pref", "show_buttons_icons", remmina_pref.show_buttons_icons);
-	g_key_file_set_integer(gkeyfile, "remmina_pref", "show_menu_icons", remmina_pref.show_menu_icons);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "auto_scroll_step", remmina_pref.auto_scroll_step);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "hostkey", remmina_pref.hostkey);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_fullscreen", remmina_pref.shortcutkey_fullscreen);
@@ -736,18 +800,21 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color14", remmina_pref.color_pref.color14 ? remmina_pref.color_pref.color14 : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color15", remmina_pref.color_pref.color15 ? remmina_pref.color_pref.color15 : "");
 
-	g_key_file_set_boolean(gkeyfile, "usage_stats", "periodic_usage_stats_permission_asked", remmina_pref.periodic_usage_stats_permission_asked);
 	g_key_file_set_boolean(gkeyfile, "usage_stats", "periodic_usage_stats_permitted", remmina_pref.periodic_usage_stats_permitted);
 	g_key_file_set_int64(gkeyfile, "usage_stats", "periodic_usage_stats_last_sent", remmina_pref.periodic_usage_stats_last_sent);
+	g_key_file_set_int64(gkeyfile, "remmina_news", "periodic_rmnews_last_get", remmina_pref.periodic_rmnews_last_get);
 	g_key_file_set_string(gkeyfile, "usage_stats", "periodic_usage_stats_uuid_prefix",
 			remmina_pref.periodic_usage_stats_uuid_prefix ? remmina_pref.periodic_usage_stats_uuid_prefix : "");
+	/* Default settings */
+	g_key_file_set_string(gkeyfile, "remmina", "name", "");
+	g_key_file_set_integer(gkeyfile, "remmina", "ignore-tls-errors", 1);
 
 	content = g_key_file_to_data(gkeyfile, &length, NULL);
 	g_file_set_contents(remmina_pref_file, content, length, &error);
 
 	if (error != NULL)
 	{
-		g_print ("%s\n", error->message);
+		g_warning ("remmina_pref_save error: %s", error->message);
 		g_clear_error (&error);
 		g_key_file_free(gkeyfile);
 		g_free(content);
@@ -959,8 +1026,7 @@ void remmina_pref_set_value(const gchar *key, const gchar *value)
 	g_free(content);
 }
 
-gchar*
-remmina_pref_get_value(const gchar *key)
+gchar* remmina_pref_get_value(const gchar *key)
 {
 	TRACE_CALL(__func__);
 	GKeyFile *gkeyfile;
@@ -973,3 +1039,17 @@ remmina_pref_get_value(const gchar *key)
 
 	return value;
 }
+
+gboolean remmina_pref_get_boolean(const gchar *key)
+{
+	TRACE_CALL(__func__);
+	GKeyFile *gkeyfile;
+	gboolean value;
+
+	gkeyfile = g_key_file_new();
+	g_key_file_load_from_file(gkeyfile, remmina_pref_file, G_KEY_FILE_NONE, NULL);
+	value = g_key_file_get_boolean(gkeyfile, "remmina_pref", key, NULL);
+	g_key_file_free(gkeyfile);
+
+	return value;
+}
